/*
  spamdyke -- a filter for stopping spam at connection time.
  Copyright (C) 2007 Sam Clippinger (samc (at) silence (dot) org)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#ifndef SPAMDYKE_H
#define SPAMDYKE_H

#ifdef TLS

#include <openssl/ssl.h>

#define VERSION_STRING                  "2.6.0+TLS"

#else /* TLS */

#define VERSION_STRING                  "2.6.0"

#endif /* TLS */

#define STRLEN(X)                       (sizeof(X) - 1)

#define DEFAULT_PATH                    "/bin:/usr/bin:/usr/local/bin:/var/qmail/bin:/home/vpopmail/bin"
#define DEFAULT_TIMEOUT_COMMAND_SECS    60
#define DEFAULT_TIMEOUT_CONNECTION_SECS 0
#define TIMEOUT_CHECKPASSWORD_SECS      30
#define TIMEOUT_TLS_SHUTDOWN_SECS       5

#define ENVIRONMENT_DELIMITER           '='
#define ENVIRONMENT_REMOTE_IP           "TCPREMOTEIP"
#define ENVIRONMENT_REMOTE_NAME         "TCPREMOTEHOST"
#define ENVIRONMENT_REMOTE_INFO         "TCPREMOTEINFO"
#define ENVIRONMENT_PATH                "PATH"
#define ENVIRONMENT_ALLOW_RELAY         "RELAYCLIENT"
#define ENVIRONMENT_HOSTNAME_TCPSERVER  "TCPLOCALHOST"
#define ENVIRONMENT_HOSTNAME_LINUX      "HOSTNAME"
#define ENVIRONMENT_HOSTNAME            (char *[]){ ENVIRONMENT_HOSTNAME_TCPSERVER, ENVIRONMENT_HOSTNAME_LINUX, NULL }
#define STRLEN_ENVIRONMENT_HOSTNAME     (int []){ STRLEN(ENVIRONMENT_HOSTNAME_TCPSERVER), STRLEN(ENVIRONMENT_HOSTNAME_LINUX), -1 }

#define TCPRULES_INFO                   '@'
#define TCPRULES_ENVIRONMENT            ':'
#define TCPRULES_NAME                   '='
#define TCPRULES_ALLOW                  "allow"
#define TCPRULES_DENY                   "deny"
#define TCPRULES_DELIMITER              ','

#define SMTP_AUTH_SPACER                ','

#define STDIN_FD                        0
#define STDOUT_FD                       1
#define STDERR_FD                       2
#define CHECKPASSWORD_FD                3

#define MKDIR_MODE                      0700
#define CHMOD_MODE                      0600

#define MAX_ADDRESS                     511
#define MAX_PATH                        4095
#define MAX_BUF                         4095
#define MAX_FILE_BUF                    65535
#define MAX_FILE_LINES                  65536
#define MAX_POLICY                      100
#define MAX_RDNS                        29
#define MAX_IP                          15
#define MAX_CHECKPASSWORD               512
#define MAX_RAND_SEED                   65536
#define RDNS_SUFFIX                     ".in-addr.arpa"
#define MINVAL(a,b)                     (((a) > (b)) ? (b) : (a))
#define MAXVAL(a,b)                     (((a) < (b)) ? (b) : (a))

#define FILTER_MASK_PASS                0x03
#define FILTER_FLAG_PASS                0x00
#define FILTER_FLAG_INTERCEPT           0x01
#define FILTER_FLAG_QUIT                0x02

#define FILTER_MASK_CHILD               0x04
// CHILD_CONTINUE is implied in assignments if CHILD_QUIT is not specified.
#define FILTER_FLAG_CHILD_CONTINUE      0x00
#define FILTER_FLAG_CHILD_QUIT          0x04

#define FILTER_MASK_AUTH                0x18
// AUTH_NONE is implied in assignments if another flag is not specified.
#define FILTER_FLAG_AUTH_NONE           0x08
#define FILTER_FLAG_AUTH_ADD            0x08
#define FILTER_FLAG_AUTH_CAPTURE        0x10

#define FILTER_MASK_TLS                 0x60
// TLS_NONE is implied in assignments if another flag is not specified.
#define FILTER_FLAG_TLS_NONE            0x00
#define FILTER_FLAG_TLS_ADD             0x20
#define FILTER_FLAG_TLS_CAPTURE         0x40
#define FILTER_FLAG_TLS_REMOVE          0x60

/*
#define FILTER_INTERCEPT_AND_CHILD_QUIT 0
#define FILTER_INTERCEPT_AND_CONTINUE   1
#define FILTER_PASS                     2
#define FILTER_QUIT                     3
#define FILTER_PASS_ADD_AUTH            4
#define FILTER_PASS_CAPTURE_AUTH        5
#define FILTER_PASS_ADD_TLS             6
#define FILTER_PASS_ADD_AUTH_ADD_TLS    7
#define FILTER_PASS_CAPTURE_TLS         8
#define FILTER_PASS_REMOVE_TLS          9
#define FILTER_PASS_ADD_AUTH_REMOVE_TLS 10
*/

#define FILTER_DECISION_ERROR           -2
#define FILTER_DECISION_UNDECIDED       -1
#define FILTER_DECISION_DO_NOT_FILTER   0
#define FILTER_DECISION_DO_FILTER       1
#define FILTER_DECISION_CHECK_AUTH      2
#define FILTER_DECISION_BYPASS_FILTER   3
#define FILTER_DECISION_PASS_TLS        4

#define GRAYLIST_NONE                   0
#define GRAYLIST_FILTER_ALL             1
#define GRAYLIST_FILTER_NONE            2

#define TLS_STATE_INACTIVE              0
#define TLS_STATE_ACTIVE_SPAMDYKE       1
#define TLS_STATE_ACTIVE_PASSTHROUGH    2

#define LOCALHOST_IP                    "127.0.0.1"
#define LOCALHOST_NAME                  "localhost"
#define MISSING_LOCAL_SERVER_NAME       "unknown.server.unknown.domain"

#define ALPHABET_FILENAME               "abcdefghijklmnopqrstuvwxyz0123456789@_-.ABCDEFGHIJKLMNOPQRSTUVWXYZ"
#define REPLACEMENT_FILENAME            '_'
#define ALPHABET_BASE64                 "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/"
#define PAD_BASE64                      '='

#define REJECT_SEVERITY_NONE            "250 "
#define REJECT_SEVERITY_HELO            "220 "
#define REJECT_SEVERITY_QUIT            "221 "
#define REJECT_SEVERITY_AUTH_SUCCESS    "235 "
#define REJECT_SEVERITY_AUTH_CHALLENGE  "334 "
#define REJECT_SEVERITY_TEMPORARY       "421 "
#define REJECT_SEVERITY_AUTH_UNKNOWN    "504 "
#define REJECT_SEVERITY_AUTH_FAILURE    "535 "
#define REJECT_SEVERITY_PERMANENT       "554 "
#define REJECT_SEVERITY_TLS_SUCCESS     "220 "
#define REJECT_SEVERITY_TLS_FAILURE     "454 "
#define STRLEN_REJECT_SEVERITY          4
#define REJECT_CRLF                     "\r\n"

#define SMTP_HELO                       "HELO"
#define SMTP_EHLO                       "EHLO"
#define SMTP_AUTH                       "AUTH"
#define SMTP_TLS                        "STARTTLS"
#define SMTP_MAIL_FROM                  "MAIL FROM"
#define SMTP_RCPT_TO                    "RCPT TO"
#define SMTP_DATA                       "DATA"
#define SMTP_DATA_END                   "."
#define SMTP_QUIT                       "QUIT"
#define SMTP_RSET                       "RSET"

#define SMTP_AUTH_UNKNOWN               -1
#define SMTP_AUTH_LOGIN                 0
#define SMTP_AUTH_PLAIN                 1
#define SMTP_AUTH_CRAM_MD5              2
#define SMTP_AUTH_TYPE_LOGIN            "LOGIN"
#define SMTP_AUTH_TYPE_PLAIN            "PLAIN"
#define SMTP_AUTH_TYPE_CRAM_MD5         "CRAM-MD5"
#define SMTP_AUTH_TYPES                 (char *[]){ SMTP_AUTH_TYPE_LOGIN, SMTP_AUTH_TYPE_PLAIN, SMTP_AUTH_TYPE_CRAM_MD5, NULL }
#define STRLEN_SMTP_AUTH_TYPES          (int []){ STRLEN(SMTP_AUTH_TYPE_LOGIN), STRLEN(SMTP_AUTH_TYPE_PLAIN), STRLEN(SMTP_AUTH_TYPE_CRAM_MD5), -1 }
#define SMTP_AUTH_ENCRYPTION            (int []){ 0, 0, 1, -1 }

#define SMTP_AUTH_LOGIN_CHALLENGE_1             "VXNlcm5hbWU6"
#define SMTP_AUTH_LOGIN_CHALLENGE_2             "UGFzc3dvcmQ6"

#define SMTP_AUTH_ORIGIN_NONE           0
#define SMTP_AUTH_ORIGIN_SPAMDYKE       1
#define SMTP_AUTH_ORIGIN_CHILD          2

#define SMTP_AUTH_STATE_NONE                    0
#define SMTP_AUTH_STATE_CMD_SEEN                1
#define SMTP_AUTH_STATE_CHALLENGE_1_SENT        2
#define SMTP_AUTH_STATE_RESPONSE_1_SEEN         3
#define SMTP_AUTH_STATE_CHALLENGE_2_SENT        4
#define SMTP_AUTH_STATE_RESPONSE_2_SEEN         5

#define SMTP_CONTINUATION               '-'
#define SMTP_STR_CONTINUATION           "-"
#define SMTP_STR_DONE                   " "
#define SMTP_EHLO_SUCCESS               "250"
#define SMTP_EHLO_AUTH                  "AUTH "
#define SMTP_EHLO_TLS                   "STARTTLS"
#define SMTP_EHLO_AUTH_INSERT_ENCRYPTION        "AUTH LOGIN PLAIN CRAM-MD5\r\n"
#define SMTP_EHLO_AUTH_INSERT_CLEAR     "AUTH LOGIN PLAIN\r\n"
#define SMTP_EHLO_TLS_INSERT            "STARTTLS\r\n"
#define SMTP_EHLO_NOTHING_INSERT        "X-NOTHING\r\n"
#define SMTP_AUTH_CHALLENGE             "334"
#define SMTP_AUTH_SUCCESS               "235"
#define SMTP_TLS_SUCCESS                "220"
#define SMTP_TLS_REGREETING             "220 ESMTP\r\n"

#define CHILD_QUIT                      ".\r\nQUIT\r\n"

#define SENDER_ADDRESS_NONE             "_none"

#define SHORT_SUCCESS                   "ALLOWED"

#define ERROR_COMMAND_LINE_NO_COMMAND   "ERROR: Missing qmail-smtpd command\n"
#define ERROR_COMMAND_LINE_UNKNOWN_OPTION       "ERROR: Unknown or incomplete option: %s\n"

#define ERROR_URL                       " See: "

struct rejection_data
  {
  char *reject_severity;
  char *reject_message;
  int strlen_reject_message;
  char *short_reject_message;
  };

#define SUCCESS_AUTH                    { REJECT_SEVERITY_AUTH_SUCCESS, "Proceed.", 8, SHORT_SUCCESS "_AUTHENTICATED" }
#define SUCCESS_TLS                     { REJECT_SEVERITY_TLS_SUCCESS, "Proceed.", 8, SHORT_SUCCESS "_TLS" }

#define ERROR_FAILURE_TLS               "Failed to negotiate TLS connection."
#define FAILURE_TLS                     { REJECT_SEVERITY_TLS_FAILURE, ERROR_FAILURE_TLS, STRLEN(ERROR_FAILURE_TLS), "FAILED_TLS" }

#define REJECTION_RCPT_TO               0
#define ERROR_RCPT_TO                   "Too many recipients. Try the remaining addresses again later."
#define REJECTION_DATA_RCPT_TO          { REJECT_SEVERITY_TEMPORARY, ERROR_RCPT_TO, STRLEN(ERROR_RCPT_TO), "DENIED_TOO_MANY_RECIPIENTS" }

#define REJECTION_RCPT_TO_LOCAL         1
#define ERROR_RCPT_TO_LOCAL             "Improper recipient address. Try supplying a domain name."
#define REJECTION_DATA_RCPT_TO_LOCAL    { REJECT_SEVERITY_PERMANENT, ERROR_RCPT_TO_LOCAL, STRLEN(ERROR_RCPT_TO_LOCAL), "DENIED_UNQUALIFIED_RECIPIENT" }

#define REJECTION_GRAYLISTED            2
#define ERROR_GRAYLISTED                "Your address has been graylisted. Try again later."
#define REJECTION_DATA_GRAYLISTED       { REJECT_SEVERITY_TEMPORARY, ERROR_GRAYLISTED, STRLEN(ERROR_GRAYLISTED), "DENIED_GRAYLISTED" }

#define REJECTION_RDNS_MISSING          3
#define ERROR_RDNS_MISSING              "Refused. You have no reverse DNS entry."
#define REJECTION_DATA_RDNS_MISSING     { REJECT_SEVERITY_TEMPORARY, ERROR_RDNS_MISSING, STRLEN(ERROR_RDNS_MISSING), "DENIED_RDNS_MISSING" }

#define REJECTION_RDNS_RESOLVE          4
#define ERROR_RDNS_RESOLVE              "Refused. Your reverse DNS entry does not resolve."
#define REJECTION_DATA_RDNS_RESOLVE     { REJECT_SEVERITY_TEMPORARY, ERROR_RDNS_RESOLVE, STRLEN(ERROR_RDNS_RESOLVE), "DENIED_RDNS_RESOLVE" }

#define REJECTION_IP_IN_NAME_CC         5
#define ERROR_IP_IN_NAME_CC             "Refused. Your reverse DNS entry contains your IP address and a country code."
#define REJECTION_DATA_IP_IN_NAME_CC    { REJECT_SEVERITY_PERMANENT, ERROR_IP_IN_NAME_CC, STRLEN(ERROR_IP_IN_NAME_CC), "DENIED_IP_IN_CC_RDNS" }

#define REJECTION_IP_IN_NAME            6
#define ERROR_IP_IN_NAME                "Refused. Your reverse DNS entry contains your IP address and a banned keyword."
#define REJECTION_DATA_IP_IN_NAME       { REJECT_SEVERITY_PERMANENT, ERROR_IP_IN_NAME, STRLEN(ERROR_IP_IN_NAME), "DENIED_IP_IN_RDNS" }

#define REJECTION_EARLYTALKER           7
#define ERROR_EARLYTALKER               "Refused. You are not following the SMTP protocol."
#define REJECTION_DATA_EARLYTALKER      { REJECT_SEVERITY_PERMANENT, ERROR_EARLYTALKER, STRLEN(ERROR_EARLYTALKER), "DENIED_EARLYTALKER" }

#define REJECTION_BLACKLIST_NAME        8
#define ERROR_BLACKLIST_NAME            "Refused. Your domain name is blacklisted."
#define REJECTION_DATA_BLACKLIST_NAME   { REJECT_SEVERITY_PERMANENT, ERROR_BLACKLIST_NAME, STRLEN(ERROR_BLACKLIST_NAME), "DENIED_BLACKLIST_NAME" }

#define REJECTION_BLACKLIST_IP          9
#define ERROR_BLACKLIST_IP              "Refused. Your IP address is blacklisted."
#define REJECTION_DATA_BLACKLIST_IP     { REJECT_SEVERITY_PERMANENT, ERROR_BLACKLIST_IP, STRLEN(ERROR_BLACKLIST_IP), "DENIED_BLACKLIST_IP" }

#define REJECTION_TIMEOUT               10
#define ERROR_TIMEOUT                   "Timeout. Talk faster next time."
#define REJECTION_DATA_TIMEOUT          { REJECT_SEVERITY_TEMPORARY, ERROR_TIMEOUT, STRLEN(ERROR_TIMEOUT), "TIMEOUT" }

#define REJECTION_SENDER_BLACKLISTED    11
#define ERROR_SENDER_BLACKLISTED        "Refused. Your sender address has been blacklisted."
#define REJECTION_DATA_SENDER_BLACKLISTED       { REJECT_SEVERITY_PERMANENT, ERROR_SENDER_BLACKLISTED, STRLEN(ERROR_SENDER_BLACKLISTED), "DENIED_SENDER_BLACKLISTED" }

#define REJECTION_RECIPIENT_BLACKLISTED 12
#define ERROR_RECIPIENT_BLACKLISTED             "Refused. Mail is not being accepted at this address."
#define REJECTION_DATA_RECIPIENT_BLACKLISTED    { REJECT_SEVERITY_PERMANENT, ERROR_RECIPIENT_BLACKLISTED, STRLEN(ERROR_RECIPIENT_BLACKLISTED), "DENIED_RECIPIENT_BLACKLISTED" }

#define REJECTION_SENDER_NO_MX          13
#define ERROR_SENDER_NO_MX              "Refused. The domain of your sender address has no mail exchanger (MX)."
#define REJECTION_DATA_SENDER_NO_MX     { REJECT_SEVERITY_TEMPORARY, ERROR_SENDER_NO_MX, STRLEN(ERROR_SENDER_NO_MX), "DENIED_SENDER_NO_MX" }

#define REJECTION_RBL                   14
#define ERROR_RBL                       "Refused. Your IP address is listed in the RBL at %s"
#define REJECTION_DATA_RBL              { REJECT_SEVERITY_PERMANENT, ERROR_RBL, STRLEN(ERROR_RBL) - 2, "DENIED_RBL_MATCH" }

#define REJECTION_SMTP_AUTH_FAILURE             15
#define ERROR_SMTP_AUTH_FAILURE                 "Refused. Authentication failed."
#define REJECTION_DATA_SMTP_AUTH_FAILURE        { REJECT_SEVERITY_AUTH_FAILURE, ERROR_SMTP_AUTH_FAILURE, STRLEN(ERROR_SMTP_AUTH_FAILURE), "FAILED_AUTH" }

#define REJECTION_SMTP_AUTH_UNKNOWN             16
#define ERROR_SMTP_AUTH_UNKNOWN                 "Refused. Unknown authentication method."
#define REJECTION_DATA_SMTP_AUTH_UNKNOWN        { REJECT_SEVERITY_AUTH_UNKNOWN, ERROR_SMTP_AUTH_UNKNOWN, STRLEN(ERROR_SMTP_AUTH_UNKNOWN), "UNKNOWN_AUTH" }

#define REJECTION_ACCESS_DENIED                 17
#define ERROR_ACCESS_DENIED                     "Refused. Access is denied."
#define REJECTION_DATA_ACCESS_DENIED            { REJECT_SEVERITY_PERMANENT, ERROR_ACCESS_DENIED, STRLEN(ERROR_ACCESS_DENIED), "DENIED_ACCESS_DENIED" }

#define REJECTION_RELAYING_DENIED               18
#define ERROR_RELAYING_DENIED                   "Refused. Sending to remote addresses (relaying) is not allowed."
#define REJECTION_DATA_RELAYING_DENIED          { REJECT_SEVERITY_PERMANENT, ERROR_RELAYING_DENIED, STRLEN(ERROR_RELAYING_DENIED), "DENIED_RELAYING" }

#define REJECTION_DATA                  (struct rejection_data []){ \
                                        REJECTION_DATA_RCPT_TO, \
                                        REJECTION_DATA_RCPT_TO_LOCAL, \
                                        REJECTION_DATA_GRAYLISTED, \
                                        REJECTION_DATA_RDNS_MISSING, \
                                        REJECTION_DATA_RDNS_RESOLVE, \
                                        REJECTION_DATA_IP_IN_NAME_CC, \
                                        REJECTION_DATA_IP_IN_NAME, \
                                        REJECTION_DATA_EARLYTALKER, \
                                        REJECTION_DATA_BLACKLIST_NAME, \
                                        REJECTION_DATA_BLACKLIST_IP, \
                                        REJECTION_DATA_TIMEOUT, \
                                        REJECTION_DATA_SENDER_BLACKLISTED, \
                                        REJECTION_DATA_RECIPIENT_BLACKLISTED, \
                                        REJECTION_DATA_SENDER_NO_MX, \
                                        REJECTION_DATA_RBL, \
                                        REJECTION_DATA_SMTP_AUTH_FAILURE, \
                                        REJECTION_DATA_SMTP_AUTH_UNKNOWN, \
                                        REJECTION_DATA_ACCESS_DENIED, \
                                        REJECTION_DATA_RELAYING_DENIED \
                                        }

#define SYSLOG_IDENTIFIER               "spamdyke"
#define SYSLOG_MISSING_DATA             "(unknown)"
#define SYSLOG_ERROR_GRAYLIST_FILE      "ERROR: cannot write to graylist file "
#define SYSLOG_ERROR_MOVE_DESCRIPTORS   "ERROR: unable to move file descriptors: "
#define SYSLOG_ERROR_FORK               "ERROR: unable to fork: "
#define SYSLOG_ERROR_PIPE               "ERROR: unable to create pipe: "
#define SYSLOG_ERROR_EXEC               "ERROR: unable to execute child process "
#define SYSLOG_ERROR_OPEN_KEYWORDS      "ERROR: unable to open keywords file "
#define SYSLOG_ERROR_OPEN_SEARCH        "ERROR: unable to open search file "
#define SYSLOG_ERROR_RES_INIT           "ERROR: unable to initialize resolver library."
#define SYSLOG_ERROR_DN_EXPAND          "ERROR: dn_expand failed for %s; this could indicate a problem with the nameserver."
#define SYSLOG_ERROR_DN_SKIPNAME        "ERROR: dn_skipname failed for %s; this could indicate a problem with the nameserver."
#define SYSLOG_ERROR_TYPE_TXT           "ERROR: dns query of %s did not include a TXT record as requested; this could indicate a problem with the name server."
#define SYSLOG_ERROR_TYPE_PTR           "ERROR: dns query of %s did not include a PTR record as requested; this could indicate a problem with the name server."
#define SYSLOG_ERROR_TYPE_MX            "ERROR: dns query of %s did not include an MX record as requested; this could indicate a problem with the name server."
#define SYSLOG_ERROR_DNS_RESPONSE       "ERROR: bad or invalid dns response to %s; this could indicate a problem with the name server."
#define SYSLOG_ERROR_STAT               "ERROR: unable to stat graylist folder "
#define SYSLOG_ERROR_MKDIR              "ERROR: unable to create graylist folder "
#define SYSLOG_ERROR_OPEN_LOG           "ERROR: unable to open traffic log file "
#define SYSLOG_ERROR_REALLOC            "ERROR: out of memory - unable to allocate %lu bytes"
#define SYSLOG_ERROR_FILE_TOO_LONG      "ERROR: ignoring file content past line %d: "
#define SYSLOG_ERROR_TLS_INIT           "ERROR: unable to initialize SSL/TLS library"
#define SYSLOG_ERROR_TLS_CERTIFICATE    "ERROR: unable to load SSL/TLS certificate from file: "
#define SYSLOG_ERROR_TLS_PRIVATEKEY     "ERROR: unable to load or decrypt SSL/TLS private key from file: "
#define SYSLOG_ERROR_TLS_ACCEPT         "ERROR: unable to start SSL/TLS connection"
#define SYSLOG_ERROR_TLS_NO_ERROR       "Operation failed but no error was reported by the SSL/TLS library."
#define SYSLOG_ERROR_TLS_ZERO_RETURN    "The connection was unexpectedly ended/closed"
#define SYSLOG_ERROR_TLS_RECALL         "The SSL/TLS library wants a function to be called again to complete. This shouldn't happen"
#define SYSLOG_ERROR_TLS_SYSCALL        "The operation failed due to an I/O error"
#define SYSLOG_ERROR_TLS_LIBRARY        "A protocol or library failure occurred"
#define SYSLOG_ERROR_TLS_EOF_FOUND      "Unexpected EOF found"

#define LOG_ACTION_TLS_PASSTHROUGH_START        -6
#define LOG_ACTION_AUTH_FAILURE                 -5
#define LOG_ACTION_AUTH_SUCCESS                 -4
#define LOG_ACTION_TLS_START                    -3
#define LOG_ACTION_TLS_END                      -2
#define LOG_ACTION_NONE                         -1
#define LOG_ACTION_REMOTE_FROM                  0
#define LOG_ACTION_CHILD_FROM                   1
#define LOG_ACTION_CHILD_FROM_DISCARDED         2
#define LOG_ACTION_FILTER_FROM                  3
#define LOG_ACTION_FILTER_TO                    4
#define LOG_ACTION_PREFIX                       (char *[]){ ">>> ", "<<< ", "<XX ", "<FF ", "FF>" }
#define LOG_ACTION_PREFIX_NONE                  ""
#define LOG_ACTION_PREFIX_TLS_SPAMDYKE          " TLS"
#define LOG_ACTION_PREFIX_TLS_PASSTHROUGH       " TLS_PASSTHROUGH"
#define LOG_ACTION_PREFIX_AUTH                  " AUTH:"
#define LOG_MESSAGE_TLS_PASSTHROUGH_START       "TLS passthrough started"
#define LOG_MESSAGE_TLS_START                   "TLS negotiated and started"
#define LOG_MESSAGE_TLS_END                     "TLS ended and closed"
#define LOG_MESSAGE_AUTH_SUCCESS                "Authentication successful"
#define LOG_MESSAGE_AUTH_FAILURE                "Authentication failed: "

struct filter_settings
  {
  char server_name[MAX_BUF + 1];
  int strlen_server_name;
  char *server_ip;
  int strlen_server_ip;
  char *local_server_name;

  char *graylist_dir;
  long graylist_min_secs;
  long graylist_max_secs;
  int max_rcpt_to;
  int use_syslog;
  char *policy_location;
  int strlen_policy_location;
  char *log_dir;
  char *blacklist_sender_path;
  char *blacklist_recipient_path;

  char **access_list_path;
  char **local_domains_path;
  char **blacklist_rdns_keyword_path;
  char **blacklist_rdns_directory_path;
  char **blacklist_ip_path;
  char **whitelist_rdns_path;
  char **whitelist_ip_path;
  char **dnsrbl_fqdn;
  char **graylist_never_ip_path;
  char **graylist_never_rdns_path;
  char **graylist_always_ip_path;
  char **graylist_always_rdns_path;
  char **smtp_auth_command;
  int smtp_auth_encryption;

  int graylist_type;
  int check_ip_in_rdns_cc;
  int check_earlytalker;
  int check_rdns_exist;
  int check_rdns_resolve;
  int check_sender_mx;

  int timeout_connection;
  int timeout_command;

  int allow_relay;
  int inside_data;
  int num_rcpt_to;
  int local_sender;

  char sender_address[MAX_ADDRESS + 1];
  char recipient_address[MAX_ADDRESS + 1];
  char graylist_path[MAX_PATH + 1];

  struct rejection_data *rejection;
  struct rejection_data *transient_rejection;
  struct rejection_data rejection_buf;
  char reject_message_buf[MAX_BUF + 1];

  char **child_argv;
  int filter_action;

  int smtp_auth_state;
  int smtp_auth_type;
  int smtp_auth_origin;
  char smtp_auth_challenge[MAX_BUF + 1];
  char smtp_auth_response[MAX_BUF + 1];
  char smtp_auth_username[MAX_BUF + 1];

  int tls_state;
#ifdef TLS
  SSL_CTX *tls_context;
  SSL *tls_session;
  char *tls_certificate_path;
  char *tls_privatekey_path;
  char *tls_privatekey_password;
#endif // TLS
  };

extern struct rejection_data *rejection;

#endif // SPAMDYKE_H
