/*
  spamdyke -- a filter for stopping spam at connection time.
  Copyright (C) 2007 Sam Clippinger (samc (at) silence (dot) org)

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License version 2 as
  published by the Free Software Foundation.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include "spamdyke.h"
#include "usage.h"

void usage(int full_text, char *error_pattern, ...)
  {
  va_list tmp_va;

  printf(
    "spamdyke %s (C)2007 Sam Clippinger, samc (at) silence (dot) org\n"
    "http://freesoftware.silence.org/\n"
    , VERSION_STRING
    );

  if (error_pattern != NULL)
    {
    printf("\n");

    va_start(tmp_va, error_pattern);
    vprintf(error_pattern, tmp_va);
    va_end(tmp_va);

    printf(
      "\n"
      "Use -h for an option summary or see README.txt for complete option details.\n"
      );
    }
  else if (full_text)
    printf(
      "\n"
      "USAGE: spamdyke [ OPTIONS ] [ -- ] qmail_smtpd_command [ qmail_smtpd_arguments ]\n"
      "\n"
      "-a MAX_RCPTS\n"
      "--max-recipients MAX_RCPTS\n"
      "  Allow a maximum of MAX_RCPTS recipients per connection for non-local senders.\n"
      "  Default: unlimited recipients per connection\n"
      "\n"
      "-b RDNS_BLACKLIST_DIR\n"
      "--rdns-blacklist-dir RDNS_BLACKLIST_DIR\n"
      "  Search RDNS_BLACKLIST_DIR directory for the remote server's rDNS name.  The\n"
      "  path for a given name can be discovered using the domain2path tool (also see\n"
      "  README.txt for details). If it is found, reject the connection. May be given\n"
      "  multiple times. Default: no rDNS blacklist\n"
      "\n"
      "-B\n"
      "--ip-blacklist-file IP_BLACKLIST_FILE\n"
      "  Search IP_BLACKLIST_FILE for the IP address of the remote server. If it is\n"
      "  found, reject the connection. May be given multiple times. Default: no IP\n"
      "  blacklist\n"
      "\n"
      "-c\n"
      "--reject-ip-in-cc-rdns\n"
      "  Search the remote server's rDNS name for its IP address. If it is found AND\n"
      "  the rDNS name ends in a two-character country code, reject the connection.\n"
      "  Default: do not search\n"
      "\n"
      "-d LOCAL_DOMAINS_FILE\n"
      "--local-domains-file LOCAL_DOMAINS_FILE\n"
      "  Use LOCAL_DOMAINS_FILE as a list of locally hosted domains (used by other\n"
      "  features to determine if an email address is local). May be given multiple\n"
      "  times. Default: no domains are local\n"
      "\n"
      "-e DELAY_SECS\n"
      "--greeting-delay-secs DELAY_SECS\n"
      "  Delay the SMTP greeting banner DELAY_SECS seconds to see if the remote server\n"
      "  begins sending data early. If it does, reject the connection. Default: no\n"
      "  delay\n"
      "\n"
      "-g GRAYLIST_DIR\n"
      "--graylist-dir GRAYLIST_DIR\n"
      "  Graylist all connections except those excluded by --never-graylist-ip-file\n"
      "  and/or --never-graylist-rdns-file and use GRAYLIST_DIR as a base for graylist\n"
      "  files. Each recipient must have a subdirectory named with their domain (e.g.\n"
      "  /.../graylist/example.com) or graylisting will not take place for that\n"
      "  recipient. See README.txt for full details. This option is mutually exclusive\n"
      "  with -G. Default: no graylisting\n"
      "\n"
      "--never-graylist-ip-file IP_FILE\n"
      "  Never graylist connections from remote servers whose IP addresses match\n"
      "  pattern(s) in IP_FILE. This option only has effect when -g is given; it is\n"
      "  ignored when -G is given. Default: do not exclude any connections from\n"
      "  graylisting\n"
      "\n"
      "--never-graylist-rdns-file RDNS_FILE\n"
      "  Never graylist connections from remote servers whose rDNS names match\n"
      "  pattern(s) in RDNS_FILE. This option only has effect when -g is given; it is\n"
      "  ignored when -G is given. Default: do not exclude any connections from\n"
      "  graylisting\n"
      "\n"
      "-G GRAYLIST_DIR\n"
      "--no-graylist-dir GRAYLIST_DIR\n"
      "  Graylist no connections except those included by --always-graylist-ip-file\n"
      "  and/or --always-graylist-rdns-file and use GRAYLIST_DIR as a base for graylist\n"
      "  files. Each recipient must have a subdirectory named with their domain (e.g.\n"
      "  /.../graylist/example.com) or graylisting will not take place for that\n"
      "  recipient. See README.txt for full details. This option is mutually exclusive\n"
      "  with -g. Default: no graylisting\n"
      "\n"
      "--always-graylist-ip-file IP_FILE\n"
      "  Always graylist connections from remote servers whose IP addresses match\n"
      "  pattern(s) in IP_FILE. This option only has effect when -G is given; it is\n"
      "  ignored when -g is given. Default: do not graylist\n"
      "\n"
      "--always-graylist-rdns-file RDNS_FILE\n"
      "  Always graylist connections from remote servers whose rDNS names match\n"
      "  pattern(s) in RDNS_FILE. This option only has effect when -G is given; it is\n"
      "  ignored when -g is given. Default: do not graylist\n"
      "\n"
      "-k KEYWORD_FILE\n"
      "--ip-in-rdns-keyword-file KEYWORD_FILE\n"
      "  Search the remote server's rDNS name for its IP address. If it is found AND it\n"
      "  contains a keyword in KEYWORD_FILE, reject the connection. May be given\n"
      "  multiple times. Default: do not search\n"
      "\n"
      "-l\n"
      "--use-syslog\n"
      "  Log mail activity to syslog. Very handy for tracking and troubleshooting.\n"
      "  Default: do not log activity, only log errors\n"
      "\n"
      "-L LOG_DIR\n"
      "--full-log-dir LOG_DIR\n"
      "  Log all SMTP traffic to files in LOG_DIR. Handy for troubleshooting delivery\n"
      "  problems but not meant to be used long-term. This option imposes a performance\n"
      "  penalty! Default: do not log all traffic\n"
      "\n"
      "-m MIN_SECS\n"
      "--graylist-min-secs MIN_SECS\n"
      "  Require a graylist entry to be present for MIN_SECS seconds before allowing\n"
      "  mail. Requires -g. Default: no minimum time required; mail is allowed after\n"
      "  the initial graylisting.\n"
      "\n"
      "-M MAX_SECS\n"
      "--graylist-max-secs MAX_SECS\n"
      "  Invalidate and remove graylist entries after they are MAX_SECS seconds old.\n"
      "  Requires -g. Default: no maximum age for graylist entries\n"
      "\n"
      "-r\n"
      "--reject-empty-rdns\n"
      "  Reject the connection if no rDNS name exists for the remote server. Default:\n"
      "  do not check for an rDNS name\n"
      "\n"
      "-R\n"
      "--reject-unresolvable-rdns\n"
      "  Reject the connection if the remote server's rDNS name does not resolve\n"
      "  (search for an A record). Default: do not attempt to resolve\n"
      "\n"
      "-s SENDER_BLACKLIST_FILE\n"
      "--sender-blacklist-file SENDER_BLACKLIST_FILE\n"
      "  Search SENDER_BLACKLIST_FILE for the sender's address. If it is found,\n"
      "  reject the connection. May be given multiple times. Default: no sender\n"
      "  blacklist\n"
      "\n"
      "-S RECIPIENT_BLACKLIST_FILE\n"
      "--recipient-blacklist-file RECIPIENT_BLACKLIST_FILE\n"
      "  Search RECIPIENT_BLACKLIST_FILE for the recipient's address. If it is found,\n"
      "  reject the connection. May be given multiple times. Default: no recipient\n"
      "  blacklist\n"
      "\n"
      "-t TIMEOUT_SECS\n"
      "--connection-timeout-secs TIMEOUT_SECS\n"
      "  Forcibly disconnect after a total of MAX_CONNECTION_SECONDS seconds,\n"
      "  regardless of activity. A value of 0 disables this feature. Default: %d\n"
      "\n"
      "-T TIMEOUT_SECS\n"
      "--idle-timeout-secs TIMEOUT_SECS\n"
      "  Forcibly disconnect after TIMEOUT_SECS seconds of inactivity. A value of 0\n"
      "  disables this feature. Default: %d\n"
      "\n"
      "-w RDNS_WHITELIST_FILE\n"
      "--rdns-whitelist-file RDNS_WHITELIST_FILE\n"
      "  Search RDNS_WHITELIST_FILE for the base domain name of the remote server's\n"
      "  rDNS name. If it is found, bypass all other tests and accept the connection.\n"
      "  May be given multiple times. Default: no whitelist\n"
      "\n"
      "-W IP_WHITELIST_FILE\n"
      "--ip-whitelist-file IP_WHITELIST_FILE\n"
      "  Search IP_WHITELIST_FILE for the IP address of the remote server. If it is\n"
      "  found, bypass all other tests and accept the connection. May be given multiple\n"
      "  times. Default: no whitelist\n"
      "\n"
      "-u POLICY_URL\n"
      "--policy-url POLICY_URL\n"
      "  Append POLICY_URL to the rejection message to explain why the rejection\n"
      "  occurred. Most servers hide rejection messages from their users and most users\n"
      "  don't read bounce messages. Maximum %d characters. Default: no policy URL\n"
      "\n"
      "-x DNSRBL\n"
      "--check-dnsrbl DNSRBL\n"
      "  Check the remote server's IP address against the realtime blackhole list\n"
      "  DNSRBL and reject the connection if it is found. May be given multiple times.\n"
      "  Default: do not check any DNSRBLs\n"
      "\n"
      "--reject-missing-sender-mx\n"
      "  Check the sender's domain name's DNS for an MX or A record. If none are found,\n"
      "  reject the connection. Requires -d. Default: do not check for MX or A records\n"
      "\n"
      "--smtp-auth-command COMMAND\n"
      "  Perform SMTP AUTH verification using COMMAND. Only cleartext authentication\n"
      "  methods will be advertised unless qmail offers encrypted ones. If the\n"
      "  authentication is valid, bypass all filters. May be given multiple times.\n"
      "  Default: do not check authentication.\n"
      "\n"
      "--smtp-auth-command-encryption COMMAND\n"
      "  Same as --smtp-auth-command but COMMAND supports encrypted authentication (e.g.\n"
      "  vpopmail's vchkpw). --smtp-auth-command and --smtp-auth-command-encryption\n"
      "  should not both be used (valid but makes no sense). May be given multiple\n"
      "  times. Default: do not check authentication.\n"
      "\n"
      "--hostname FQDN\n"
      "  Use FQDN as the fully qualified domain name of this host. This value is used\n"
      "  during SMTP AUTH challenge-response. Default: %s\n"
      "\n"
      "--access-file TCP_ACCESS_FILE\n"
      "  Use TCP_ACCESS_FILE to determine if the remote host can connect and/or relay.\n"
      "  Only really needed if using spamdyke SMTP AUTH (but qmail isn't patched).  See\n"
      "  the README file for an explanation.\n"
      "\n"

#ifdef TLS

      "--tls-certificate-file CERTIFICATE_FILE\n"
      "  Offer and handle TLS using the SSL certificate in CERTIFICATE_FILE. The file\n"
      "  must be in PEM format. If CERTIFICATE_FILE does not contain the private key,\n"
      "  --tls-privatekey-file must be used. Default: do not offer TLS and pass any\n"
      "  TLS traffic through to qmail without interference.\n"
      "\n"
      "--tls-privatekey-file PRIVATEKEY_FILE\n"
      "  Use PRIVATEKEY_FILE for the TLS certificate's private key, overriding the key\n"
      "  found in CERTIFICATE_FILE (given with --tls-certificate-file), if it contains\n"
      "  one. PRIVATEKEY_FILE must be in PEM format. Requires --tls-certificate-file.\n"
      "\n"
      "--tls-privatekey-password PASSWORD\n"
      "  Use PASSWORD to decrypt the SSL private key found in CERTIFICATE_FILE (given\n"
      "  with --tls-certificate-file) or in PRIVATEKEY_FILE (given with\n"
      "  --tls-privatekey-file), if necessary. NOTE: using this option reveals the\n"
      "  password to anyone who can view a process list on the server! Requires\n"
      "  --tls-certificate-file or --tls-privatekey-file.\n"
      "\n"

#endif /* TLS */

      "See README.txt for a complete explanation of these options.\n"
      , DEFAULT_TIMEOUT_CONNECTION_SECS
      , DEFAULT_TIMEOUT_COMMAND_SECS
      , MAX_POLICY
      , MISSING_LOCAL_SERVER_NAME
      );

  exit(0);

  return;
  }
